##############################################################################
##############################################################################
#####                            Ventana.tcl
##############################################################################
##### Creates the window which will provide some information to the user 
##### while a file is being downloaded.
##############################################################################
##### Copyright 1999 Andrs Garca Garca
##### The contents of this file are distributed under the terms of the GNU
##### license
##############################################################################

namespace eval Ventana {

source [file join $dirGetleft scripts Rizo.tcl]

##############################################################################
# Recortar_cadena
#    Recorta el nombre del fichero o del enlace para que cuadre con el espacio
#    que hay para escribirlos.
#
# Parmetros
#    cadena: cadena con el nombre a recortar
#    long: longitud en la que tiene que quedar
#
# Devuelve:
#    La cadena recortada
##############################################################################
proc Recortar_cadena {cadena long} {

    set longitud_cadena [string length $cadena]
    if {$longitud_cadena>$long} {
        set index [expr round($long/2)-2]
        set fin   [expr $longitud_cadena-$index]
        set primer [string range $cadena 0 $index]
        set segun  [string range $cadena $fin end]
        append tmp $primer "..." $segun
        return $tmp
    } else {
        return $cadena
    }
}

##############################################################################
# Bajando_archivo
#   Crea una bonita ventana con informacin de cmo va la bajada de un fichero
#
# Parmetros:
#    fichero: fichero donde se va a guardar el fichero a bajar
#    enlace: url del fichero.
##############################################################################
proc Bajando_archivo {fichero enlace} {
    global Ventana::Rizo::meta Ventana::Rizo::mensaje_error
    global file link directorio_base tcl_platform coord

    set bajando  [toplevel .bajando]
    wm title $bajando "Downloading file"

    fijar_coordenadas
    wm geometry $bajando 290x155+[expr $coord(x)+100]+[expr $coord(y)+50]
    wm resizable $bajando 0 0

    set file $fichero
    set link $enlace

    set marco     [frame $bajando.marco]
    set marco_in  [frame $bajando.marco.marco_interno]
    set marco_url [frame $marco_in.url]
    set marco_fic [frame $marco_in.fic]
    set marco_por [frame $marco_in.por]
    set marco_que [frame $marco_in.que]
    set marco_bar [frame $marco_in.bar]
    set botones   [frame $bajando.botones]

    set etq_url [label $marco_url.etq_url -text "url: "]
    set etq_fic [label $marco_fic.etq_fic -text "File: "]
    set etq_por [label $marco_por.etq_por -text "%: "]

    set Ventana::Rizo::informe(cual) "Left: "
    set etq_que [label $marco_que.etq_que -textvariable Ventana::Rizo::informe(cual)]

    set Ventana::Rizo::informe(porcentaje)   0
    set Ventana::Rizo::informe(fin)          0
    set Ventana::Rizo::informe(largo)        1
    set Ventana::Rizo::informe(queda)        ""
    set Ventana::Rizo::informe(detener)      0

    if [regexp {Windows} $tcl_platform(os)] {
        set anchoEnt 48
        set anchoEtq 44
    } else {
        set anchoEnt 37
        set anchoEtq 33
    }

    set etq_url2 [label $marco_url.etq_url2 -text [Recortar_cadena $enlace $anchoEnt] \
         -bd 2 -relief sunken -width 45 -anchor w]

    regexp (?:$directorio_base)(.*) [file rootname $fichero] nada tmp

    set etq_fic2 [label $marco_fic.etq_fic2 -text [Recortar_cadena $tmp $anchoEtq] \
        -anchor w]
    set etq_por2 [label $marco_por.etq_por2 -textvariable Ventana::Rizo::informe(porcentaje) -width 5 -anchor w]
    set etq_que2 [label $marco_que.etq_que2 -textvariable Ventana::Rizo::informe(queda) -width 30 -anchor w]

    set barra [canvas $marco_bar.lienzo -width 275 -height 10 -bg white -bd 2 -relief sunken]

    set pausa [button $botones.pausa -text Pause    \
        -command "Ventana::Bajando_parar pausa $fichero $enlace" -width 8]
    set cancelar [button $botones.cancelar -text Cancel \
        -command "set Ventana::Rizo::informe(parar) 1" -width 8]

    pack $marco   -side top    -fill x -expand true
    pack $botones -side bottom -fill x -expand true -padx 7
    pack $marco_in -padx 10 -pady 10
    pack $marco_url $marco_in $marco_fic $marco_por $marco_que $marco_bar -fill x -expand true
    pack $etq_url $etq_url2 -side left
    pack $etq_fic $etq_fic2 -side left
    pack $etq_por $etq_por2 -side left
    pack $etq_que $etq_que2 -side left
    pack $barra

    pack $cancelar $pausa -side right -padx 3 -pady 5

    tkwait visibility $barra

    Ventana::Rizo::Datos $fichero $enlace

    Barra_actualizar $barra

    return
}

############################################################################
# Bajando_encabezamiento
#    Solicita al servidor el encabezamiento de la pgina
#
# Parmetros
#    La pgina a bajar
############################################################################
proc Bajando_encabezamiento {enlace} {
    set Ventana::Rizo::informe(parar) 0
    Ventana::Rizo::Cabeza $enlace
    tkwait variable Ventana::Rizo::informe(parar)
}

#############################################################################
# Bajando_parar
#    Dirige las operaciones en caso de que se cancele una bajada.
#
# Parmetros:
#    cual: 'pausa' se hace una pausa temporal.
#          'cancelar' se para definitivamente todo el proceso.
#    fichero: fichero en el que se est guardando.
#    enlace: url que se est bajando.
#############################################################################
proc Bajando_parar {cual fichero enlace} {

    switch $cual {
        pausa {
            set Ventana::Rizo::informe(detener) 1
            set boton ".bajando.botones.pausa"
            $boton configure -text "Resume" -command "
                set Ventana::Rizo::informe(detener) 0
                .bajando.botones.pausa configure -text Pause \
                    -command \"Bajando_parar pausa $fichero $enlace\"
                .bajando.marco.marco_interno.bar.lienzo create rect 0 0 280 15 -fill white
                Ventana::Rizo::Reanudar $fichero $enlace
                Ventana::Barra_actualizar .bajando.marco.marco_interno.bar.lienzo
            "
        }
    }
    return
}

#############################################################################
# Barra_actualizar
#    Actualiza la barra de progreso.
#
# Parmetros:
#    barra: path de la barra de progreso
#############################################################################
proc Barra_actualizar {barra} {
    global Ventana::Rizo::informe

    if [string match $informe(porcentaje) ""] return
    set hecho [expr 2.75 * $informe(porcentaje)]

    if {($informe(largo)==0)} {
        set Ventana::Rizo::informe(cual) "Got: "
    } else {
        set Ventana::Rizo::informe(cual) "Left: "
    }
    $barra create rect 0 0 $hecho 15 -fill blue
    if {$informe(detener)==0} {
        if {($informe(fin)==0) && ($informe(parar)!=1)} {
            after 1000 Ventana::Barra_actualizar $barra
        } else {
            destroy .bajando
        }
    }
    return
}

}